@@ -3,7 +3,9 @@ import logging
 import os
 import platform
 import subprocess
-from typing import Any, Dict, List, Optional
+import json
+import tempfile
+import typing
 
 import pandas
 
@@ -16,9 +18,7 @@ from pydelica.solutions import SolutionHandler
 
 
 class Session:
-    def __init__(
-        self,
-        log_level: OMLogLevel = OMLogLevel.NORMAL) -> None:
+    def __init__(self, log_level: OMLogLevel = OMLogLevel.NORMAL) -> None:
         """Session object which handles model runs.
 
         This class is used to initiate model runs, building the models and
@@ -29,24 +29,23 @@ class Session:
         log_level : OMLogLevel, optional
             level of Modelica logging, by default OMLogLevel.NORMAL
         """
-        self._solutions: Dict[str, SolutionHandler] = {}
-        self._model_parameters: Dict[str, Model] = {}
-        self._simulation_opts: Dict[str, SimulationOptions] = {}
-        self._binaries: Dict[str, str] = {}
+        self._solutions: typing.Dict[str, SolutionHandler] = {}
+        self._model_parameters: typing.Dict[str, Model] = {}
+        self._simulation_opts: typing.Dict[str, SimulationOptions] = {}
+        self._current_profile: typing.Optional[typing.Dict] = None
+        self._current_info: typing.Optional[typing.Dict] = None
+        self._binaries: typing.Dict[str, str] = {}
         self._logger: logging.Logger = logging.getLogger("PyDelica")
         if log_level == OMLogLevel.DEBUG:
             self._logger.setLevel(logging.DEBUG)
         self._compiler = Compiler()
         self._log_level = log_level
         self._session_directory = os.getcwd()
-        self._assert_fail_level = 'error'
+        self._assert_fail_level = "error"
 
     def use_library(
-        self,
-        library_name: str,
-        library_version: str = "",
-        library_directory: str = ""
-        ) -> None:
+        self, library_name: str, library_version: str = "", library_directory: str = ""
+    ) -> None:
         """Specify version of a library to use
 
         Parameters
@@ -58,7 +57,9 @@ class Session:
         library_directory : str, optional
             location of library on system, by default OM default paths
         """
-        self._compiler._library.use_library(library_name, library_version, library_directory)
+        self._compiler._library.use_library(
+            library_name, library_version, library_directory
+        )
 
     def fail_on_assert_level(self, assert_level: str) -> None:
         """Change assertion level on which model execution should fail
@@ -75,7 +76,7 @@ class Session:
         assert_level : str
             new level of assertion to trigger failure
         """
-        if assert_level in {'info', 'warning', 'error', 'debug', 'never'}:
+        if assert_level in {"info", "warning", "error", "debug", "never"}:
             self._assert_fail_level = assert_level
         else:
             raise pde.UnknownOptionError(
@@ -83,14 +84,32 @@ class Session:
                 "not a valid option"
             )
 
+    @property
+    def code_profile(self) -> typing.Optional[typing.Dict]:
+        return self._current_profile
+
+    @property
+    def code_info(self) -> typing.Optional[typing.Dict]:
+        return self._current_info
+
+    def _recover_profile(self, build_dir: str, run_dir: str) -> None:
+        """Recovers a profile file if one exists"""
+        _prof_files = glob.glob(os.path.join(run_dir, "*_prof.json"))
+        _info_files = glob.glob(os.path.join(build_dir, "*_info.json"))
+        if _prof_files:
+            self._current_profile = json.load(open(_prof_files[0]))
+        if _info_files:
+            self._current_info = json.load(open(_info_files[0]))
 
     def build_model(
         self,
         modelica_source_file: str,
         model_addr: str = None,
-        extra_models: Optional[List[str]] = None,
+        extra_models: typing.Optional[typing.List[str]] = None,
         c_source_dir: str = None,
+        profiling: str = None,
         update_input_paths_to: str = None,
+        omc_build_flags: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None
     ):
         """Build a Modelica model from a source file
 
@@ -100,12 +119,16 @@ class Session:
             Modelica source file to compile
         model_addr : str, optional
             address of model within source file, else default
-        extra_models : Optional[List[str]], optional
+        extra_models : typing.Optional[typing.List[str]], optional
             additional models required for compile, by default None
         c_source_dir : str, optional
             directory containing any additional required C sources, by default None
+        profiling : str, optional
+            if set, activates the OMC profiling at the specified level
         update_input_paths_to : str, optional
             update input paths within model to another location, by default None
+        omc_build_flags : typing.Optional[typing.Dict[str, typing.Optional[str]]]
+            additional flags to pass to the OMC compiler
 
         Raises
         ------
@@ -114,17 +137,28 @@ class Session:
         RuntimeError
             If model compilation failed
         """
+        if not omc_build_flags:
+            omc_build_flags = {}
+
         self._logger.debug(
             "Building model %sfrom file '%s'",
-            model_addr + " " if model_addr else "",
+            f'{model_addr} ' if model_addr else "",
             modelica_source_file,
         )
+
+
+        for flag, value in omc_build_flags.items():
+            self._compiler.set_omc_flag(flag, value)
+
+        self._compiler.set_profile_level(profiling)
+
         _binary_loc = self._compiler.compile(
             modelica_source_file=modelica_source_file,
             model_addr=model_addr,
             extra_models=extra_models,
-            c_source_dir=c_source_dir
+            c_source_dir=c_source_dir,
         )
+
         _xml_files = glob.glob(os.path.join(_binary_loc, "*.xml"))
 
         if not _xml_files:
@@ -132,12 +166,11 @@ class Session:
                 "Failed to retrieve XML files from model compilation"
             )
 
+
         self._logger.debug("Parsing generated XML files")
         for xml_file in _xml_files:
             _model_name = os.path.basename(xml_file).split("_init")[0]
-            self._model_parameters[_model_name] = Model(
-                modelica_source_file, xml_file
-            )
+            self._model_parameters[_model_name] = Model(modelica_source_file, xml_file)
 
             _binary = _model_name
 
@@ -171,9 +204,7 @@ class Session:
             # i.e. .mat/.csv to point to the correct location as an
             # absolute path
             if update_input_paths_to:
-                self._set_input_files_directory(
-                    _model_name, update_input_paths_to
-                )
+                self._set_input_files_directory(_model_name, update_input_paths_to)
 
             # Allows easy creation of a Pandas dataframe for displaying solutions
             self._set_output_to_csv()
@@ -185,7 +216,10 @@ class Session:
             )
         return self._binaries[model_name]
 
-    def get_parameters(self, model_name: str = None) -> Dict[str, Any]:
+    def get_parameters(
+        self,
+        model_name: str = None
+        ) -> typing.Union[Model, typing.Dict[str, typing.Any]]:
         """Retrieve a full parameter list
 
         Parameters
@@ -195,7 +229,7 @@ class Session:
 
         Returns
         -------
-        Dict[str, Any]
+        typing.Dict[str, typing.Any]
             dictionary containing parameters by name and their values
         """
         if model_name:
@@ -204,7 +238,7 @@ class Session:
 
             return self._model_parameters[model_name]
         else:
-            _out_params: Dict[str, Any] = {}
+            _out_params: typing.Dict[str, typing.Any] = {}
             for model in self._model_parameters:
                 for param in self._model_parameters[model]:
                     if param in _out_params:
@@ -212,7 +246,7 @@ class Session:
                     _out_params[param] = self._model_parameters[model][param]
         return _out_params
 
-    def get_parameter(self, param_name: str) -> Any:
+    def get_parameter(self, param_name: str) -> typing.Any:
         """Retrieve the value of a specific parameter
 
         Parameters
@@ -222,7 +256,7 @@ class Session:
 
         Returns
         -------
-        Any
+        typing.Any
             the value of the parameter specified
         """
         for model in self._model_parameters:
@@ -238,8 +272,8 @@ class Session:
                 return self._model_parameters[model].get_parameter(param_name)
         raise pde.UnknownParameterError(param_name)
 
-    def get_simulation_options(self, model_name: str = None) -> Dict[str, Any]:
-        """Retrieve Dictionary of the Simulation Options
+    def get_simulation_options(self, model_name: str = None) -> SimulationOptions:
+        """Retrieve typing.Dictionary of the Simulation Options
 
         Parameters
         ----------
@@ -248,7 +282,7 @@ class Session:
 
         Returns
         -------
-        Dict[str, Any]
+        SimulationOptions
             dictionary containing all simulation options
 
         Raises
@@ -262,9 +296,7 @@ class Session:
             raise pde.UnknownModelError(model_name)
         return self._simulation_opts[model_name]
 
-    def get_simulation_option(
-        self, option: str, model_name: str = None
-    ) -> Any:
+    def get_simulation_option(self, option: str, model_name: str = None) -> typing.Any:
         """Retrieve a single option for a given model.
 
         Parameters
@@ -276,7 +308,7 @@ class Session:
 
         Returns
         -------
-        Any
+        typing.Any
             value for the given option
 
         Raises
@@ -287,20 +319,18 @@ class Session:
             if the given option name is not recognised
         """
         if not model_name:
-            return self._simulation_opts[list(self._binaries.keys())[0]][
-                option
-            ]
+            return self._simulation_opts[list(self._binaries.keys())[0]][option]
         else:
             return self._simulation_opts[model_name][option]
 
-    def set_parameter(self, param_name: str, value: Any) -> None:
+    def set_parameter(self, param_name: str, value: typing.Any) -> None:
         """Set a parameter to a given value
 
         Parameters
         ----------
         param_name : str
             name of model parameter to update
-        value : Any
+        value : typing.Any
             new value to assign to the given parameters
         """
         if isinstance(value, dict):
@@ -339,10 +369,9 @@ class Session:
                 _addr = os.path.join(input_dir, value["value"])
                 self._model_parameters[model_name].set_parameter(param, _addr)
 
-    def simulate(self,
-        model_name: str = None,
-        verbosity: Optional[OMLogLevel]=None
-        ) -> None:
+    def simulate(
+        self, model_name: str = None, verbosity: typing.Optional[OMLogLevel] = None
+    ) -> None:
         """Run simulation using the built models
 
         Parameters
@@ -370,9 +399,7 @@ class Session:
 
         _binary_dir = os.path.dirname(self._binaries[model_name])
 
-        _args = [
-            self._binaries[model_name], f'-inputPath={_binary_dir}'
-        ]
+        _args = [self._binaries[model_name], f"-inputPath={_binary_dir}"]
 
         if not os.path.exists(self._binaries[model_name]):
             raise pde.BinaryNotFoundError(
@@ -385,20 +412,24 @@ class Session:
         elif self._log_level.value:
             _args += [self._log_level.value]
 
-        _run_sim = subprocess.run(
-            _args,
-            stderr=subprocess.PIPE,
-            stdout=subprocess.PIPE,
-            text=True,
-            shell=False,
-        )
+        with tempfile.TemporaryDirectory() as run_dir:
 
-        pde.parse_error_string_simulate(
-            _run_sim.stdout,
-            self._assert_fail_level
-        )
+            _run_sim = subprocess.run(
+                _args,
+                stderr=subprocess.PIPE,
+                stdout=subprocess.PIPE,
+                text=True,
+                shell=False,
+                cwd=run_dir
+            )
 
-        self._solutions[model_name].retrieve_session_solutions()
+            pde.parse_error_string_simulate(_run_sim.stdout, self._assert_fail_level)
+
+            if _run_sim.returncode != 0:
+                raise pde.OMExecutionError(f"[{_run_sim.returncode}] Simulation failed with:\n{_run_sim.stderr}")
+
+            self._solutions[model_name].retrieve_session_solutions(run_dir)
+            self._recover_profile(os.path.dirname(self._binaries[model_name]), run_dir)
 
     def _set_output_to_csv(self):
         for model in self._simulation_opts:
@@ -406,9 +437,7 @@ class Session:
 
     def set_solver(self, solver: Solver, model_name: str = None) -> None:
         if model_name:
-            self._simulation_opts[model_name].set_option(
-                "solver", solver.value
-            )
+            self._simulation_opts[model_name].set_option("solver", solver.value)
         else:
             for model in self._simulation_opts:
                 self._simulation_opts[model].set_option("solver", solver.value)
@@ -421,46 +450,32 @@ class Session:
     ) -> None:
         if model_name:
             if start_time:
-                self._simulation_opts[model_name].set_option(
-                    "startTime", start_time
-                )
+                self._simulation_opts[model_name].set_option("startTime", start_time)
             if stop_time:
-                self._simulation_opts[model_name].set_option(
-                    "stopTime", stop_time
-                )
+                self._simulation_opts[model_name].set_option("stopTime", stop_time)
         else:
             for model in self._simulation_opts:
                 if start_time:
-                    self._simulation_opts[model].set_option(
-                        "startTime", start_time
-                    )
+                    self._simulation_opts[model].set_option("startTime", start_time)
                 if stop_time:
-                    self._simulation_opts[model].set_option(
-                        "stopTime", stop_time
-                    )
+                    self._simulation_opts[model].set_option("stopTime", stop_time)
 
     def set_tolerance(self, tolerance: float, model_name: str = None) -> None:
         if model_name:
-            self._simulation_opts[model_name].set_option(
-                "tolerance", tolerance
-            )
+            self._simulation_opts[model_name].set_option("tolerance", tolerance)
         else:
             for model in self._simulation_opts:
                 self._simulation_opts[model].set_option("tolerance", tolerance)
 
     def set_variable_filter(self, filter_str: str, model_name: str = None) -> None:
         if model_name:
-            self._simulation_opts[model_name].set_option(
-                "variableFilter", filter_str
-            )
+            self._simulation_opts[model_name].set_option("variableFilter", filter_str)
         else:
             for model in self._simulation_opts:
-                self._simulation_opts[model].set_option(
-                    "variableFilter", filter_str
-                )
+                self._simulation_opts[model].set_option("variableFilter", filter_str)
 
     def set_simulation_option(
-        self, option_name: str, value: Any, model_name: str = None
+        self, option_name: str, value: typing.Any, model_name: str = None
     ) -> None:
         if model_name:
             self._simulation_opts[model_name].set_option(option_name, value)
@@ -476,7 +491,7 @@ class Session:
 
         Returns
         -------
-        Dict
+        typing.Dict
             dictionary containing outputs to all simulated models as Pandas
             dataframes
 
