@@ -6,12 +6,13 @@ import logging
 import glob
 import re
 import platform
+import shutil
 import defusedxml.ElementTree as ET
 
 from pydelica.exception import (
     UnknownOptionError,
     UnknownLibraryError,
-    NotImplementedError
+    NotImplementedError,
 )
 
 from typing import Any, Iterable, List
@@ -25,6 +26,7 @@ class Solver(enum.Enum):
 
 class LibrarySetup:
     """Object containing setup for a particular Modelica library version"""
+
     def __init__(self) -> None:
         """Create a setup object
 
@@ -37,28 +39,33 @@ class LibrarySetup:
         self._libraries = []
         self._session_library = None
 
-        # Disable environment setting on Windows as it does not work at present
-        if platform.system() != "Windows":
-            self._libraries = self._get_system_libraries()
-            self._session_library = tempfile.mkdtemp()
-            self._copy_libraries()
+        self._libraries = self._get_system_libraries()
+        self._session_library = tempfile.mkdtemp()
+        self._copy_libraries()
+
 
     def _copy_libraries(self) -> None:
         for library in self._libraries:
-            _dest = os.path.join(
-                self._session_library, os.path.basename(library)
-            )
+            _dest = os.path.join(self._session_library, os.path.basename(library))
 
             if not os.path.exists(_dest):
-                os.symlink(library, _dest)
+                # Symlinking only works on UNIX systems
+                if platform.system() != "Windows":
+                    os.symlink(library, _dest)
+                elif os.path.isdir(library):
+                    shutil.copytree(library, _dest, symlinks=True)
+                else:
+                    shutil.copyfile(library, _dest)
 
-    def use_library(self, library_name: str, version: str, library_directory: str = "") -> None:
+    def use_library(
+        self, library_name: str, version: str, library_directory: str = ""
+    ) -> None:
         """Use a specific library version
 
         This function looks through all library directories (symlinks) and
         checks which match the requested library. The version number of the
-        directory is then compared to requested version and us symlinked, other
-        versions are unlinked.
+        directory is then compared to requested version and is symlinked
+        (or copied if Windows), other versions are unlinked (or removed).
 
         Parameters
         ----------
@@ -74,11 +81,6 @@ class LibrarySetup:
         UnknownLibraryError
             if the library and version are not recognised
         """
-        if platform.system() == "Windows":
-            raise NotImplementedError(
-                "No support yet for library version setting in Windows"
-            )
-
         _test_str = f"{library_name} {version}"
         self._logger.debug(f"Selecting Library '{_test_str}'")
 
@@ -89,7 +91,7 @@ class LibrarySetup:
             )
 
         if library_directory:
-            _libraries = glob.glob(os.path.join(library_directory, '*'))
+            _libraries = glob.glob(os.path.join(library_directory, "*"))
         else:
             _libraries = self._libraries
 
@@ -99,65 +101,90 @@ class LibrarySetup:
             if library_name.lower() not in library.lower():
                 continue
 
-            # Create the address for the symlink
-            _new_lib = os.path.join(
-                self._session_library, os.path.basename(library)
-            )
+            # Create the address for the symlink/destination
+            _new_lib = os.path.join(self._session_library, os.path.basename(library))
 
             # if no space in library filename then assume no version number
-            if ' ' not in os.path.basename(library):
+            if " " not in os.path.basename(library):
                 continue
 
             _name, _info = os.path.basename(library).split()
-            _version = re.findall(
-                r'[0-9]+\.[0-9]+\.[0-9]+',
-                _info
-            )
+
+            _version = re.findall(r"[0-9]+\.[0-9]+\.[0-9]+", _info)
 
             # If the split name is length 1 this means there is no
             # version string anyway
             if not _version:
                 continue
-            
-            _version = _version[0]
+
+            _version: str = _version[0]
 
             # Check that the name matches the requested library name
             if library_name.lower().strip() == _name.lower().strip():
                 # Check the version matches the requested version
-                # if it does ensure this is symlinked, if not unlink it
+                # if it does ensure this is symlinked/copied,
+                # if not unlink/remove it
                 if version.lower().strip() == _version.lower().strip():
                     if not os.path.exists(_new_lib):
-                        self._logger.debug(f"Linking: {library} -> {_new_lib}")
-                        os.symlink(library, _new_lib)
+                        if platform.system() != "Windows":
+                            self._logger.debug(f"Linking: {library} -> {_new_lib}")
+                            os.symlink(library, _new_lib)
+                        elif os.path.isdir(library):
+                            # Libraries are directories in Windows
+                            self._logger.debug(f"Copying: {library} -> {_new_lib}")
+                            shutil.copytree(library, _new_lib, symlinks=True)
+                        else:
+                            # Libraries are directories in Windows
+                            self._logger.debug(f"Copying: {library} -> {_new_lib}")
+                            shutil.copyfile(library, _new_lib)
                 elif os.path.exists(_new_lib):
-                    self._logger.debug(
-                        f"Unlinking: {_new_lib} -> {library}"
-                    )
-                    os.unlink(_new_lib)
+                    if platform.system() != "Windows":
+                        self._logger.debug(f"Unlinking: {_new_lib} -> {library}")
+                        os.unlink(_new_lib)
+                    else:
+                        self._logger.debug(f"Removing: {_new_lib}")
+                        # Libraries are directories in Windows
+                        if os.path.isdir(_new_lib):
+                            shutil.rmtree(_new_lib)
+                        else:
+                            os.remove(_new_lib)
 
     def get_library(self) -> str:
         return self._session_library
 
     def _get_system_libraries(self) -> List[str]:
         if "MODELICAPATH" in os.environ:
-            _library_dirs = os.environ["MODELICAPATH"].split(":")
+            _library_dirs = os.environ["MODELICAPATH"].split(os.pathsep)
             _libs = []
             for library_dir in _library_dirs:
                 _libs += glob.glob(os.path.join(library_dir, "*"))
         elif platform.system() == "Windows":
-            _library_dir = os.environ["OPENMODELICALIBRARY"]
-            _libs = glob.glob(os.path.join(_library_dir, "*"))
+            _home = os.environ["HOME"]
+            _user_libraries = os.path.join(
+                _home,
+                "AppData",
+                "Roaming",
+                ".openmodelica",
+                "libraries"
+            )
+            _library_dir = os.path.join(
+                os.environ["OPENMODELICAHOME"],
+                "lib", "omlibrary"
+            )
+            _libs = glob.glob(os.path.join(_user_libraries, "*"))
+            _libs += glob.glob(os.path.join(_library_dir, "*"))
         else:
             # Try typical linux locations
             _library_dirs = [
                 "/usr/lib/omlibrary",
-                f"{os.environ['HOME']}/.openmodelica/libraries"
+                f"{os.environ['HOME']}/.openmodelica/libraries",
             ]
             _libs = []
             for library_dir in _library_dirs:
                 _libs += glob.glob(os.path.join(library_dir, "*"))
         return _libs
 
+
 class SimulationOptions(collections.abc.MutableMapping):
     """
     Simulation Options
@@ -165,6 +192,7 @@ class SimulationOptions(collections.abc.MutableMapping):
 
     Object contains configuration settings for simulation within Modelica
     """
+
     def __init__(self, xml_model_file: str) -> None:
         """Create a configuration object from a given model XML file
 
@@ -196,9 +224,7 @@ class SimulationOptions(collections.abc.MutableMapping):
         _xml_obj = ET.parse(self._model_xml)
 
         for opt in _xml_obj.findall("DefaultExperiment")[0].attrib:
-            _xml_obj.findall("DefaultExperiment")[0].attrib[opt] = str(
-                self._opts[opt]
-            )
+            _xml_obj.findall("DefaultExperiment")[0].attrib[opt] = str(self._opts[opt])
 
         _xml_obj.write(self._model_xml)
 
@@ -229,9 +255,7 @@ class SimulationOptions(collections.abc.MutableMapping):
         """
         if option_name not in self._opts:
             raise UnknownOptionError(option_name)
-        _opt = [
-            i for i in self._opts.keys() if i.lower() == option_name.lower()
-        ][0]
+        _opt = [i for i in self._opts.keys() if i.lower() == option_name.lower()][0]
         self._opts[_opt] = value
         self._write_opts()
 
